#!/usr/bin/env python
"""
Test runner script.

Please use `tox` to run it in multiple environments.
"""

import django
from django.conf import settings
from django.core.management import call_command


gettext = lambda s: s
conf = dict(
    LANGUAGES = (
        ('ru', gettext('Russian')),
        ('en', gettext('English')),
    ),
    USE_TZ = False,
    INSTALLED_APPS = [
        'modeltranslation',
        'autoslug'
    ],
    DATABASES = dict(
        default = dict(
            ENGINE='django.db.backends.sqlite3',
            NAME=':memory:',
        ),
    ),
    AUTOSLUG_SLUGIFY_FUNCTION = 'django.template.defaultfilters.slugify',
)


settings.configure(**conf)
django.setup()

if __name__ == "__main__":
    call_command('test', 'autoslug')



# coding: utf-8
#
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#

# django
import datetime
from django.core.exceptions import ImproperlyConfigured, FieldDoesNotExist
from django.db.models import ForeignKey
from django.db.models.fields import DateField
from django.template.defaultfilters import slugify as django_slugify
from django.utils.timezone import localtime, is_aware

try:
    # i18n-friendly approach
    from unidecode import unidecode
except ImportError:
    try:
        # Cyrillic transliteration (primarily Russian)
        from pytils.translit import slugify
    except ImportError:
        # fall back to Django's default method
        slugify = django_slugify
else:
    # Use Django's default method over decoded string
    def slugify(value):
        return django_slugify(unidecode(value))


def get_prepopulated_value(field, instance):
    """
    Returns preliminary value based on `populate_from`.
    """
    if hasattr(field.populate_from, '__call__'):
        # AutoSlugField(populate_from=lambda instance: ...)
        return field.populate_from(instance)
    else:
        # AutoSlugField(populate_from='foo')
        attr = getattr(instance, field.populate_from)
        return callable(attr) and attr() or attr


def generate_unique_slug(field, instance, slug, manager):
    """
    Generates unique slug by adding a number to given value until no model
    instance can be found with such slug. If ``unique_with`` (a tuple of field
    names) was specified for the field, all these fields are included together
    in the query when looking for a "rival" model instance.
    """

    original_slug = slug = crop_slug(field, slug)

    default_lookups = tuple(get_uniqueness_lookups(field, instance, field.unique_with))

    index = 1

    if not manager:
        manager = field.model._default_manager


    # keep changing the slug until it is unique
    while True:
        # find instances with same slug
        lookups = dict(default_lookups, **{field.name: slug})
        rivals = manager.filter(**lookups)
        if instance.pk:
            rivals = rivals.exclude(pk=instance.pk)

        if not rivals:
            # the slug is unique, no model uses it
            return slug

        # the slug is not unique; change once more
        index += 1

        # ensure the resulting string is not too long
        tail_length = len(field.index_sep) + len(str(index))
        combined_length = len(original_slug) + tail_length
        if field.max_length < combined_length:
            original_slug = original_slug[:field.max_length - tail_length]

        # re-generate the slug
        data = dict(slug=original_slug, sep=field.index_sep, index=index)
        slug = '%(slug)s%(sep)s%(index)d' % data

        # ...next iteration...


def get_uniqueness_lookups(field, instance, unique_with):
    """
    Returns a dict'able tuple of lookups to ensure uniqueness of a slug.
    """
    for original_lookup_name in unique_with:
        if '__' in original_lookup_name:
            field_name, inner_lookup = original_lookup_name.split('__', 1)
        else:
            field_name, inner_lookup = original_lookup_name, None

        try:
            other_field = instance._meta.get_field(field_name)
        except FieldDoesNotExist:
            raise ValueError('Could not find attribute %s.%s referenced'
                             ' by %s.%s (see constraint `unique_with`)'
                             % (instance._meta.object_name, field_name,
                                instance._meta.object_name, field.name))

        if field == other_field:
            raise ValueError('Attribute %s.%s references itself in `unique_with`.'
                             ' Please use "unique=True" for this case.'
                             % (instance._meta.object_name, field_name))

        value = getattr(instance, field_name)
        if not value:
            if other_field.blank:
                field_object = instance._meta.get_field(field_name)
                if isinstance(field_object, ForeignKey):
                    lookup = '%s__isnull' % field_name
                    yield lookup, True
                break
            raise ValueError('Could not check uniqueness of %s.%s with'
                             ' respect to %s.%s because the latter is empty.'
                             ' Please ensure that "%s" is declared *after*'
                             ' all fields listed in unique_with.'
                             % (instance._meta.object_name, field.name,
                                instance._meta.object_name, field_name,
                                field.name))
        if isinstance(value, datetime.datetime) and is_aware(value):
            value = localtime(value)
        if isinstance(other_field, DateField):    # DateTimeField is a DateField subclass
            inner_lookup = inner_lookup or 'day'

            if '__' in inner_lookup:
                raise ValueError('The `unique_with` constraint in %s.%s'
                                 ' is set to "%s", but AutoSlugField only'
                                 ' accepts one level of nesting for dates'
                                 ' (e.g. "date__month").'
                                 % (instance._meta.object_name, field.name,
                                    original_lookup_name))

            parts = ['year', 'month', 'day']
            try:
                granularity = parts.index(inner_lookup) + 1
            except ValueError:
                raise ValueError('expected one of %s, got "%s" in "%s"'
                                    % (parts, inner_lookup, original_lookup_name))
            else:
                for part in parts[:granularity]:
                    lookup = '%s__%s' % (field_name, part)
                    yield lookup, getattr(value, part)
        else:
            # TODO: this part should be documented as it involves recursion
            if inner_lookup:
                if not hasattr(value, '_meta'):
                    raise ValueError('Could not resolve lookup "%s" in `unique_with` of %s.%s'
                                     % (original_lookup_name, instance._meta.object_name, field.name))
                for inner_name, inner_value in get_uniqueness_lookups(field, value, [inner_lookup]):
                    yield original_lookup_name, inner_value
            else:
                yield field_name, value


def crop_slug(field, slug):
    if field.max_length < len(slug):
        return slug[:field.max_length]
    return slug


try:
    import translitcodec
except ImportError:
    pass
else:
    import re
    PUNCT_RE = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.]+')

    def translitcodec_slugify(codec):
        def _slugify(value, delim='-', encoding=''):
            """
            Generates an ASCII-only slug.

            Borrowed from http://flask.pocoo.org/snippets/5/
            """
            if encoding:
                encoder = "%s/%s" % (codec, encoding)
            else:
                encoder = codec
            result = []
            for word in PUNCT_RE.split(value.lower()):
                word = word.encode(encoder)
                if word:
                    result.append(word)
            return unicode(delim.join(result))
        return _slugify

    translit_long = translitcodec_slugify("translit/long")
    translit_short = translitcodec_slugify("translit/short")
    translit_one = translitcodec_slugify("translit/one")



# coding: utf-8
#
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#
from django.db.models import (
    Model, CharField, DateField, DateTimeField, BooleanField, ForeignKey, Manager, CASCADE
)

# this app
from autoslug import AutoSlugField
from autoslug.settings import slugify as default_slugify


class SimpleModel(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField()


class ModelWithUniqueSlug(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='name', unique=True)


class ModelWithUniqueSlugFK(Model):
    name = CharField(max_length=200)
    simple_model = ForeignKey(SimpleModel, on_delete=CASCADE)
    slug = AutoSlugField(populate_from='name', unique_with='simple_model__name')


class ModelWithUniqueSlugDate(Model):
    date = DateField()
    slug = AutoSlugField(unique_with='date')


class ModelWithUniqueSlugDay(Model):  # same as ...Date, just more explicit
    date = DateTimeField()
    slug = AutoSlugField(unique_with='date__day')


class ModelWithUniqueSlugMonth(Model):
    date = DateField()
    slug = AutoSlugField(unique_with='date__month')


class ModelWithUniqueSlugYear(Model):
    date = DateField()
    slug = AutoSlugField(unique_with='date__year')


class ModelWithLongName(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='name')


class ModelWithLongNameUnique(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='name', unique=True)


class ModelWithCallable(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from=lambda instance: 'the %s' % instance.name)


class ModelWithCallableAttr(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='get_name')

    def get_name(self):
        return 'spam, %s and spam' % self.name


class ModelWithNullable(Model):
    name = CharField(max_length=200, blank=True, null=True)
    slug = AutoSlugField(populate_from='name', blank=True, null=True)


class ModelWithBlank(Model):
    name = CharField(max_length=200, blank=True, null=True)
    slug = AutoSlugField(populate_from='name', blank=True)


class ModelWithCustomPrimaryKey(Model):
    custom_primary_key = CharField(primary_key=True, max_length=1)
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='name', unique=True)


custom_slugify = lambda value: default_slugify(value).replace('-', '_')


class ModelWithCustomSlugifier(Model):
    slug = AutoSlugField(unique=True, slugify=custom_slugify)


class ModelWithCustomSeparator(Model):
    slug = AutoSlugField(unique=True, sep='_')


class ModelWithReferenceToItself(Model):
    slug = AutoSlugField(unique_with='slug')


class ModelWithWrongReferencedField(Model):
    slug = AutoSlugField(unique_with='wrong_field')


class ModelWithWrongLookupInUniqueWith(Model):
    slug = AutoSlugField(unique_with='name__foo')
    name = CharField(max_length=10)


class ModelWithWrongFieldOrder(Model):
    slug = AutoSlugField(unique_with='date')
    date = DateField(blank=False, null=False)


class ModelWithAcceptableEmptyDependency(Model):
    date = DateField(blank=True, null=True)
    slug = AutoSlugField(unique_with='date')


class ModelWithAutoUpdateEnabled(Model):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='name', always_update=True)


class ModelWithSlugSpaceSharedIntegrityError(ModelWithUniqueSlug):
    pass


class SharedSlugSpace(Model):
    objects = Manager()
    name = CharField(max_length=200)
    # ensure that any subclasses use the base model's manager for testing
    # slug uniqueness
    slug = AutoSlugField(populate_from='name', unique=True, manager=objects)


class ModelWithSlugSpaceShared(SharedSlugSpace):
    pass


class ModelWithUniqueSlugFKNull(Model):
    name = CharField(max_length=200)
    simple_model = ForeignKey(SimpleModel, null=True, blank=True, default=None, on_delete=CASCADE)
    slug = AutoSlugField(populate_from='name', unique_with='simple_model')


class ModeltranslationOne(Model):
    title = CharField(max_length=255)
    description = CharField(max_length=255)
    slug = AutoSlugField(populate_from='title', always_update=True, unique=True)


class NonDeletedObjects(Manager):
    def get_queryset(self):
        return super(NonDeletedObjects, self).get_queryset().filter(is_deleted=False)


class AbstractModelWithCustomManager(Model):
    is_deleted = BooleanField(default=False)

    objects = NonDeletedObjects()
    all_objects = Manager()

    class Meta:
        abstract = True

    def delete(self, using=None):
        self.is_deleted = True
        self.save()


class NonDeletableModelWithUniqueSlug(AbstractModelWithCustomManager):
    name = CharField(max_length=200)
    slug = AutoSlugField(populate_from='name', unique=True, manager_name='all_objects')



# coding: utf-8
#
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#
from modeltranslation.translator import translator, TranslationOptions
from .models import ModeltranslationOne


class ModeltranslationOneTranslation(TranslationOptions):
    fields = ('title', 'description')


translator.register(ModeltranslationOne,
                    ModeltranslationOneTranslation)



import inspect
import io
import os


__all__ = ['__version__']


# A simple `from autoslug import __version__` would trigger a cascading import
# of `autoslug.fields`, then `django`.  As the latter may not be available
# (being a dependency), we try to work around it.
#
# Cases:
# 1) building the package with a global interpreter and no dependencies
#    installed globally;
# 2) building documentation e.g. on RTFD server (no Django settings available
#    on runtime);
# 3) installing django-autoslug before Django itself (highly unlikely).
#
__version__ = None
thisfile = inspect.getfile(inspect.currentframe())
path = os.path.join(os.path.abspath(os.path.dirname(thisfile)), 'autoslug/__init__.py')
with io.open(path, encoding='utf8') as f:
    for line in f:
        if line.startswith('__version__'):
            exec(line)
            break
assert __version__, 'autoslug.__version__ must be imported correctly'



# coding: utf-8
#
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#
from autoslug.fields import AutoSlugField


__version__ = '1.9.8'
__all__ = ['AutoSlugField']



# coding: utf-8
#
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#

# python
import datetime
import sys
import unittest

# django
from django.db import IntegrityError
from django.test import TestCase
from django.test import override_settings
from django.utils.timezone import make_aware

# this package
from .models import *


PYPY_DATE_FUNC_SKIP_MSG = 'PyPy has troubles with Django date function + SQLite'


class AutoSlugFieldTestCase(TestCase):
    def test_simple_model(self):
        a = SimpleModel(name='test')
        a.save()
        assert a.slug == 'simplemodel'

    def test_unique_slug(self):
        greeting = 'Hello world!'
        a = ModelWithUniqueSlug(name=greeting)
        a.save()
        assert a.slug == u'hello-world'
        b = ModelWithUniqueSlug(name=greeting)
        b.save()
        assert b.slug == u'hello-world-2'

    def test_unique_slug_fk(self):
        sm1 = SimpleModel.objects.create(name='test')
        sm2 = SimpleModel.objects.create(name='test')
        sm3 = SimpleModel.objects.create(name='test2')
        greeting = 'Hello world!'
        a = ModelWithUniqueSlugFK.objects.create(name=greeting, simple_model=sm1)
        assert a.slug == u'hello-world'
        b = ModelWithUniqueSlugFK.objects.create(name=greeting, simple_model=sm2)
        assert b.slug == u'hello-world-2'
        c = ModelWithUniqueSlugFK.objects.create(name=greeting, simple_model=sm3)
        assert c.slug == u'hello-world'
        d = ModelWithUniqueSlugFK.objects.create(name=greeting, simple_model=sm1)
        assert d.slug == u'hello-world-3'
        sm3.name = 'test'
        sm3.save()
        assert c.slug == u'hello-world'
        c.save()
        assert c.slug == u'hello-world-4'

    def test_unique_slug_fk_null(self):
        "See issue #13"
        sm1 = SimpleModel.objects.create(name='test')
        a = ModelWithUniqueSlugFKNull.objects.create(name='test', simple_model=sm1)
        assert a.slug == u'test'

        b = ModelWithUniqueSlugFKNull.objects.create(name='test')
        assert b.slug == u'test'
        c = ModelWithUniqueSlugFKNull.objects.create(name='test', simple_model=sm1)
        assert c.slug == u'test-2'

    @unittest.skipIf('PyPy' in sys.version, PYPY_DATE_FUNC_SKIP_MSG)
    def test_unique_slug_date(self):
        a = ModelWithUniqueSlugDate(slug='test', date=datetime.date(2009, 9,  9))
        b = ModelWithUniqueSlugDate(slug='test', date=datetime.date(2009, 9,  9))
        c = ModelWithUniqueSlugDate(slug='test', date=datetime.date(2009, 9, 10))
        for m in a,b,c:
            m.save()
        assert a.slug == u'test'
        assert b.slug == u'test-2'
        assert c.slug == u'test'

    @unittest.skipIf('PyPy' in sys.version, PYPY_DATE_FUNC_SKIP_MSG)
    @override_settings(USE_TZ=True, DEFAULT_TIMEZONE='US/Pacific')
    def test_unique_slug_date_with_tz(self):
        try:
            import pytz
        except ImportError:
            raise unittest.SkipTest("pytz is not available, skipping test")
        same_day = make_aware(datetime.datetime(2009, 9,  9, 1), pytz.UTC)
        different_day = make_aware(datetime.datetime(2009, 9, 10, 1), pytz.UTC)
        a = ModelWithUniqueSlugDate(slug='test', date=same_day)
        b = ModelWithUniqueSlugDate(slug='test', date=same_day)
        c = ModelWithUniqueSlugDate(slug='test', date=different_day)
        for m in a,b,c:
            m.save()
        assert a.slug == u'test'
        assert b.slug == u'test-2'
        assert c.slug == u'test'

    @unittest.skipIf('PyPy' in sys.version, PYPY_DATE_FUNC_SKIP_MSG)
    def test_unique_slug_day(self):
        a = ModelWithUniqueSlugDay(slug='test', date=datetime.date(2009, 9,  9))
        b = ModelWithUniqueSlugDay(slug='test', date=datetime.date(2009, 9,  9))
        c = ModelWithUniqueSlugDay(slug='test', date=datetime.date(2009, 9, 10))
        for m in a,b,c:
            m.save()
        assert a.slug == u'test'
        assert b.slug == u'test-2'
        assert c.slug == u'test'

    @unittest.skipIf('PyPy' in sys.version, PYPY_DATE_FUNC_SKIP_MSG)
    def test_unique_slug_month(self):
        a = ModelWithUniqueSlugMonth(slug='test', date=datetime.date(2009, 9,  9))
        b = ModelWithUniqueSlugMonth(slug='test', date=datetime.date(2009, 9, 10))
        c = ModelWithUniqueSlugMonth(slug='test', date=datetime.date(2009, 10, 9))
        for m in a,b,c:
            m.save()
        assert a.slug == u'test'
        assert b.slug == u'test-2'
        assert c.slug == u'test'

    @unittest.skipIf('PyPy' in sys.version, PYPY_DATE_FUNC_SKIP_MSG)
    def test_unique_slug_year(self):
        a = ModelWithUniqueSlugYear(slug='test', date=datetime.date(2009, 9,  9))
        b = ModelWithUniqueSlugYear(slug='test', date=datetime.date(2009, 10, 9))
        c = ModelWithUniqueSlugYear(slug='test', date=datetime.date(2010, 9,  9))
        for m in a,b,c:
            m.save()
        assert a.slug == u'test'
        assert b.slug == u'test-2'
        assert c.slug == u'test'

    def test_long_name(self):
        long_name = 'x' * 250
        a = ModelWithLongName(name=long_name)
        a.save()
        assert len(a.slug) == 50    # original slug is cropped by field length

    def test_long_name_unique(self):
        long_name = 'x' * 250
        a = ModelWithLongNameUnique(name=long_name)
        a.save()
        assert len(a.slug) == 50    # original slug is cropped by field length
        b = ModelWithLongNameUnique(name=long_name)
        b.save()
        assert b.slug[-3:] == u'x-2'    # uniqueness is forced
        assert len(b.slug) == 50        # slug is cropped

    def test_nullable(self):
        a = ModelWithNullable.objects.create(name=None)
        assert a.slug is None

    def test_blank(self):
        a = ModelWithBlank.objects.create(name=None)
        assert a.slug is not None
        assert a.slug == ''

    def test_empty_slugify(self):
        a = ModelWithUniqueSlug.objects.create(name='?')
        assert a.slug

    def test_callable(self):
        a = ModelWithCallable.objects.create(name='larch')
        assert a.slug == u'the-larch'

    def test_callable_attr(self):
        a = ModelWithCallableAttr.objects.create(name='albatross')
        assert a.slug == u'spam-albatross-and-spam'

    def test_custom_primary_key(self):
        # just check if models are created without exceptions
        a = ModelWithCustomPrimaryKey.objects.create(custom_primary_key='a',
                                                     name='name used in slug')
        b = ModelWithCustomPrimaryKey.objects.create(custom_primary_key='b',
                                                     name='name used in slug')
        assert a.slug == u'name-used-in-slug'

    def test_custom_slugifier(self):
        a = ModelWithCustomSlugifier.objects.create(slug='hello world!')
        b = ModelWithCustomSlugifier.objects.create(slug='hello world!')
        assert b.slug == u'hello_world-2'

    def test_custom_separator(self):
        a = ModelWithCustomSeparator.objects.create(slug='hello world!')
        b = ModelWithCustomSeparator.objects.create(slug='hello world!')
        assert b.slug == u'hello-world_2'

    def test_self_reference(self):
        a = ModelWithReferenceToItself(slug='test')
        errmsg = (
            'Attribute ModelWithReferenceToItself.slug references itself '
            'in `unique_with`. Please use "unique=True" for this case.'
        )
        with self.assertRaises(ValueError, msg=errmsg):
            a.save()

    def test_wrong_referenced_field(self):
        a = ModelWithWrongReferencedField(slug='test')
        errmsg = (
            'Could not find attribute ModelWithWrongReferencedField.wrong_field '
            'referenced by ModelWithWrongReferencedField.slug (see constraint '
            '`unique_with`)'
        )
        with self.assertRaises(ValueError, msg=errmsg):
            a.save()

    def test_wrong_lookup_in_unique_with(self):
        a = ModelWithWrongLookupInUniqueWith(name='test', slug='test')
        errmsg = (
            'Could not resolve lookup "name__foo" in `unique_with`'
            ' of ModelWithWrongLookupInUniqueWith.slug'
        )
        with self.assertRaises(ValueError, msg=errmsg):
            a.save()

    def test_wrong_field_order(self):
        a = ModelWithWrongFieldOrder(slug='test')
        errmsg = (
            'Could not check uniqueness of ModelWithWrongFieldOrder.slug '
            'with respect to ModelWithWrongFieldOrder.date because the latter '
            'is empty. Please ensure that "slug" is declared *after* all '
            'fields listed in unique_with.'
        )

        with self.assertRaises(ValueError, msg=errmsg):
            a.save()

    def test_acceptable_empty_dependency(self):
        model = ModelWithAcceptableEmptyDependency
        instances = [model.objects.create(slug='hello') for x in range(0,2)]
        assert [x.slug for x in model.objects.all()] == [u'hello', u'hello-2']

    def test_auto_update_enabled(self):
        a = ModelWithAutoUpdateEnabled(name='My name')
        a.save()
        assert a.slug == u'my-name'
        a.name = 'My new name'
        a.save()
        assert a.slug == u'my-new-name'

    @unittest.expectedFailure
    def test_slug_space_shared_integrity_error(self):
        a = ModelWithUniqueSlug(name='My name')
        a.save()
        b = ModelWithSlugSpaceSharedIntegrityError(name='My name')
        with self.assertRaises(IntegrityError, msg='column slug is not unique'):
            b.save()

    def test_shared_slug_space(self):
        a = SharedSlugSpace(name='My name')
        a.save()
        assert a.slug == u'my-name'
        b = ModelWithSlugSpaceShared(name='My name')
        b.save()
        assert b.slug == u'my-name-2'

    def test_autoslug_with_manager_name(self):
        a = NonDeletableModelWithUniqueSlug.objects.create(name='My name')
        self.assertEqual(a.slug, u'my-name')
        a.delete()
        b = NonDeletableModelWithUniqueSlug.objects.create(name='My name')
        self.assertEqual(b.slug, u'my-name-2')

    def test_deconstruct_with_manager(self):
        a = SharedSlugSpace(name='TestName')
        a.save()
        _, _, _, kwargs = a._meta.get_field('slug').deconstruct()
        self.assertNotIn('manager', kwargs)


class AutoSlugModelTranslationTestCase(TestCase):

    def test_regression_33(self):
        """
        https://bitbucket.org/neithere/django-autoslug/issues/33/

        Autoslug tries to populate localized AutoSlugField that is not marked
        as translatable by modeltranslation.
        """
        a = ModeltranslationOne(title='hello', description='foo')
        a.save()



#!/usr/bin/env python
# coding: utf-8
#
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#

import io
import os
from setuptools import setup


from _version_helper import __version__


readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
    name     = 'django-autoslug',
    version  = __version__,
    packages = ['autoslug'],

    requires = ['python (>= 3.5)', 'django (>= 1.11)'],
    # in case you want to use slugify() with support for transliteration:
    extras_require = {
        'cyrillic': 'pytils >= 0.2',
        'translitcodec': 'translitcodec >= 0.3',
    },
    description  = 'An automated slug field for Django.',
    long_description = readme,
    author       = 'Justin Mayer',
    author_email = 'entrop@gmail.com',
    url          = 'https://github.com/justinmayer/django-autoslug/',
    download_url = 'https://github.com/justinmayer/django-autoslug/archive/master.zip',
    license      = 'GNU Lesser General Public License (LGPL), Version 3',
    keywords     = 'django field slug auto unique transliteration i18n',
    classifiers  = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: General',
    ],
)
